"""\
This file is an installation script for CMFReportTool.  It's meant to be
used as an External Method.  To use, add an external method to the
root of the CMF Site that you want CMFReportTool Content registered in with the
configuration:

 id: install_events
 title: Install ReportTool *optional*
 module name: CMFReportTool.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Try it' tab.  The install function will execute and give
information about the steps it took to register and install the
CMF ReportTool into the CMF Site instance. 
"""
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName

from Products.CMFReportTool import ReportTool, cmfreporttool_globals

from Acquisition import aq_base
from cStringIO import StringIO
import string

def install(self):
	" Register the CMF ReportTool with portal_types and friends "
	out = StringIO()
	
	typestool = getToolByName(self, 'portal_types')
	skinstool = getToolByName(self, 'portal_skins')

	addTool = self.manage_addProduct['CMFReportTool'].manage_addTool
	
	addTool('CMF Report Tool',None)


	# Setup the skins
	# This is borrowed from CMFDefault/scripts/addImagesToSkinPaths.pys
	if 'zpt_reporttool' not in skinstool.objectIds():
		# We need to add Filesystem Directory Views for any directories
		# in our skins/ directory.  These directories should already be
		# configured.
		addDirectoryViews(skinstool, 'skins', cmfreporttool_globals)
		out.write("Added 'zpt_reporttool' directory view to portal_skins\n")

	# Now we need to go through the skin configurations and insert
	# 'zpt_reporttool' into the configurations.  Preferably, this should be
	# right before where 'content' is placed.  Otherwise, we append
	# it to the end.
	skins = skinstool.getSkinSelections()
	for skin in skins:
		path = skinstool.getSkinPath(skin)
		path = map(string.strip, string.split(path,','))
		if 'zpt_reporttool' not in path:
			try: path.insert(path.index('content'), 'zpt_reporttool')
			except ValueError:
				path.append('zpt_reporttool')
				
			path = string.join(path, ', ')
			# addSkinSelection will replace exissting skins as well.
			skinstool.addSkinSelection(skin, path)
			out.write("Added 'zpt_reporttool' to %s skin\n" % skin)
		else:
			out.write("Skipping %s skin, 'zpt_reporttool' is already set up\n" % (
				skin))


	return out.getvalue()

