#
#
#   CMFReportTool - Generating PDF-Skins on the Fly from CMF-Sites
#
#   Copyright (C) 2002  net-labs Systemhaus GmbH
#                                               info@net-labs.de
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Portal report tool.
$Id: STXParser.py,v 1.1.1.1 2002/10/29 20:19:16 ueck Exp $
"""
__version__='$Revision: 1.1.1.1 $'[11:-2]

from string import join, split, find
from cgi import escape
import re, sys
from StructuredText import ST


class STXClass:

	element_types={
		'#text': '_text',
		'StructuredTextDocument': 'document',
		'StructuredTextParagraph': 'paragraph',
		'StructuredTextExample': 'example',
		'StructuredTextBullet': 'bullet',
		'StructuredTextNumbered': 'numbered',
		'StructuredTextDescription': 'description',
		'StructuredTextDescriptionTitle': 'descriptionTitle',
		'StructuredTextDescriptionBody': 'descriptionBody',
		'StructuredTextSection': 'section',
		'StructuredTextSectionTitle': 'sectionTitle',
		'StructuredTextLiteral': 'literal',
		'StructuredTextEmphasis': 'emphasis',
		'StructuredTextStrong': 'strong',
		'StructuredTextLink': 'link',
		'StructuredTextXref': 'xref',
		'StructuredTextInnerLink':'innerLink',
		'StructuredTextNamedLink':'namedLink',
		'StructuredTextUnderline':'underline',
		'StructuredTextTable':'table',
		'StructuredTextSGML':'sgml',
		}


	def dispatch(self, doc, level, output):
		getattr(self, self.element_types[doc.getNodeName()])(doc, level, output)
		
	def __call__(self, doc, level=1, header=1,encoding="iso-8859-1"):
		r=[]
		self.header = header
		self.encoding = encoding
		self.inParagraph = 0
		self.dispatch(doc, level-1, r.append)
		return join(r,'')

	def _text(self, doc, level, output):
		output((doc.getNodeValue().lstrip()))

	def document(self, doc, level, output):
		children=doc.getChildNodes()

		if self.header==1:
			output('<?xml version="1.0" encoding="%s" ?>\n<document filename="doc.pdf">\n' % self.encoding)
			if (children and
				 children[0].getNodeName() == 'StructuredTextSection'):
				output('<title>%s</title>\n' %
						 children[0].getChildNodes()[0].getNodeValue())
			output('<content>\n')

		for c in children:
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)

		if self.header==1:
			output('</content>\n')
			output('</document>\n')

	def section(self, doc, level, output):
		children=doc.getChildNodes()
		for c in children:
			getattr(self, self.element_types[c.getNodeName()])(c, level+1, output)
		
	def sectionTitle(self, doc, level, output):
		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="20"/>\n<para style="Heading%d">' % (level))
		else:
			output('<spacer height="30"/><para style="Heading%d">' % (level))
		self.inParagraph=1
		
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
			
		if inPara:
			output('</para><spacer height="20"/>\n<para style="Normal">')
		else:
			output('</para>\n')
		self.inParagraph=inPara


	def description(self, doc, level, output):
		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('<spacer height="20"/><para style="Normal">')
		self.inParagraph=1

		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)

		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('</para>')
		self.inParagraph = inPara
		


	def descriptionTitle(self, doc, level, output):
		output('<bullet>')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output('</bullet>\n')
		
	def descriptionBody(self, doc, level, output):
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output('\n')

	def bullet(self, doc, level, output):
		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Bullet">')
		else:
			output('<spacer height="20"/><para style="Bullet">')
		self.inParagraph=1
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('</para>')
		self.inParagraph = inPara

	def numbered(self, doc, level, output):
		p=doc.getPreviousSibling()
		if p is None or p.getNodeName() is not doc.getNodeName():			
			output('\n<seqreset id="stx" />\n')

		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('<spacer height="20"/><para style="Normal">')
		self.inParagraph=1

		output('<bullet><seq id="stx" />. </bullet>')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('</para>')
		self.inParagraph = inPara

	def example(self, doc, level, output):
		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="10"/>\n<prefmt style="Example" dedent="%d">' % (level))
		else:
			output('<spacer height="20"/><prefmt style="Example" dedent="%d">' % (level))
		self.inParagraph=1
			
		for c in doc.getChildNodes():
			output(escape(c.getNodeValue()))

		if inPara:
			output('</prefmt><spacer height="10"/>\n<para style="Normal">')
		else:
			output('</prefmt>')
		self.inParagraph = inPara


	def paragraph(self, doc, level, output):

		inPara = self.inParagraph
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('<spacer height="20"/><para style="Normal">')
		self.inParagraph = 1

		for c in doc.getChildNodes():
			if c.getNodeName() in ['StructuredTextParagraph']:
				getattr(self, self.element_types[c.getNodeName()])(
					c, level, output)
			else:
				getattr(self, self.element_types[c.getNodeName()])(
					c, level, output)
		if inPara:
			output('</para><spacer height="10"/>\n<para style="Normal">')
		else:
			output('</para>')
		self.inParagraph = inPara
	
	def link(self, doc, level, output):
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output(' (%s) ' % doc.href)

	def emphasis(self, doc, level, output):
		output('<b>')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output('</b>')


	def literal(self, doc, level, output):
		output('<font name="Courier">')
		for c in doc.getChildNodes():
			output(c.getNodeValue())
		output('</font>')

	def strong(self, doc, level, output):
		output('<b>')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output('</b>')
	 
	def underline(self, doc, level, output):
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		  
	def innerLink(self, doc, level, output):
		output(' [')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output(']')
	
	def namedLink(self, doc, level, output):
		output(' [')
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)
		output(']')
	
	def sgml(self,doc,level,output):
		for c in doc.getChildNodes():
			getattr(self, self.element_types[c.getNodeName()])(c, level, output)

	def xref(self, doc, level, output):
		val = doc.getNodeValue()
		output('%s [%s]' % (val, val) )
	
	def table(self,doc,level,output):
		"""
		A StructuredTextTable holds StructuredTextRow(s) which
		holds StructuredTextColumn(s). A StructuredTextColumn
		is a type of StructuredTextParagraph and thus holds
		the actual data.
		"""
		
		if self.inParagraph:
			output('</para><spacer height="20"/>\n')

		output('<table style="StandardTable">\n')
		for row in doc.getRows()[0]:
			output("<tr>\n")
			for column in row.getColumns()[0]:
				output('<td>')
				for c in column.getChildNodes():
					getattr(self, self.element_types[c.getNodeName()])(c, level, output)
				output("</td>\n")
			output("</tr>\n")
		output("</table>\n")

		if self.inParagraph:
			output('<spacer height="20"/>\n<para>')
		  

STXParser = STXClass()


